/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/mirror_base.hpp>
#include <mirror/pre_registered/full.hpp>
#include <mirror/stream.hpp>
#include <mirror/factory.hpp>
#include <mirror/utils/script_factory.hpp>

#include <mirror/pre_registered/class/std/vector.hpp>

#include <cassert>
#include <iostream>
#include <stdexcept>

#include "./test.hpp"

MIRROR_NAMESPACE_BEGIN

MIRROR_NAMESPACE_END

int main(void)
{
	try
	{
		using namespace mirror;
		//
		typedef std::pair<int, std::string> pair_t;
		typedef std::vector<pair_t> vector_t;
		std::string src =
		"{"
			"std::pair<int, std::string>(),"
			"std::pair<int, std::string>(1,	'blah'),"
			"std::pair<int, std::string>(2,	'buhehe'),"
			"std::pair<int, std::string>(3,	'buhaha')"
		"}";
		//
		script_factory_input in;
		script_factory_maker::factory<vector_t>::type vf = in.data();
		in.set(src);
		vector_t x(vf());
		for(auto i = x.begin(), e = x.end(); i != e; ++i)
		{
			std::cout << stream::to_json::from(
				*i, [](std::ostream& out){ out << "x"; }
			) << std::endl;
		}
		//
		script_factory_maker::factory<std::vector<std::string> >::type sf = in.data();
		std::string src2 = "{std::string(''), 'blah', 'buhehe', 'buhaha'}";
		in.set(src2);
		std::vector<std::string> y(sf());
		std::cout << y.size() << std::endl;
	}
	catch(std::exception const& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}
