/**
 *  @file mirror/example/factories/tetrahedron.hpp
 *
 *  Two classes used in some of the examples showing
 *  the use of meta-constructors and factories
 *
 *  Copyright 2008-2010 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef MIRROR_EXAMPLE_FACTORIES_TETRAHEDRON_1011291729_HPP
#define MIRROR_EXAMPLE_FACTORIES_TETRAHEDRON_1011291729_HPP

#include <mirror/meta_class.hpp>
#include <mirror/meta_enum.hpp>
#include <mirror/pre_registered/type/native.hpp>
#include <string>
#include <ctime>

namespace test {

enum class gender
{
	female,
	male
};

struct person
{
	std::string first_name;
	std::string middle_name;
	std::string family_name;

	std::tm birth_date;
	gender sex;

	float weight;
	float height;

	person(
		const std::string& _first_name,
		const std::string& _family_name,
		const std::tm _birth_date,
		gender _sex,
		float _weight,
		float _height
	): first_name(_first_name)
	 , family_name(_family_name)
	 , birth_date(_birth_date)
	 , sex(_sex)
	 , weight(_weight)
	 , height(_height)
	{ }

	person(
		const std::string& _first_name,
		const std::string& _middle_name,
		const std::string& _family_name,
		const std::tm _birth_date,
		gender _sex,
		float _weight,
		float _height
	): first_name(_first_name)
	 , middle_name(_middle_name)
	 , family_name(_family_name)
	 , birth_date(_birth_date)
	 , sex(_sex)
	 , weight(_weight)
	 , height(_height)
	{ }

	void remeasure(float _weight, float _height)
	{
		weight = _weight;
		height = _height;
	}
};

} // namespace test

MIRROR_REG_BEGIN

MIRROR_QREG_GLOBAL_SCOPE_NAMESPACE(test)

MIRROR_REG_ENUM_BEGIN(test, gender)
	MIRROR_REG_ENUM_VALUE(female)
	MIRROR_REG_ENUM_VALUE(male)
MIRROR_REG_ENUM_END

MIRROR_REG_CLASS_BEGIN(struct, test, person)
MIRROR_REG_CLASS_MEM_VARS_BEGIN
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, first_name)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, middle_name)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, family_name)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, birth_date)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, sex)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, weight)
	MIRROR_REG_CLASS_MEM_VAR(_, _, _, height)
MIRROR_REG_CLASS_MEM_VARS_END
MIRROR_REG_CONSTRUCTORS_BEGIN
	MIRROR_REG_COPY_CONSTRUCTOR(public)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, 1)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, first_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, family_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, birth_date)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, sex)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, weight)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, height)
	MIRROR_REG_CONSTRUCTOR_END(1)

	MIRROR_REG_CONSTRUCTOR_BEGIN(public, 2)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, first_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, middle_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, family_name)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, birth_date)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, sex)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, weight)
		MIRROR_REG_CONSTRUCTOR_PARAM(_, height)
	MIRROR_REG_CONSTRUCTOR_END(2)
MIRROR_REG_CONSTRUCTORS_END
MIRROR_REG_MEM_FUNCTIONS_BEGIN
	MIRROR_REG_MEM_FUNCTION_BEGIN(public, _, void, remeasure, _)
		MIRROR_REG_MEM_FUNCTION_PARAM(_, weight)
		MIRROR_REG_MEM_FUNCTION_PARAM(_, height)
	MIRROR_REG_MEM_FUNCTION_END(remeasure,_,_)
MIRROR_REG_MEM_FUNCTIONS_END
MIRROR_REG_CLASS_END

MIRROR_REG_END

#endif

